#ifndef _G_H_
#define _G_H_
// g* stuff

namespace G
{
	namespace Str
	{
		// InStr - Find occurance of substring in string
		unsigned int InStr(const char *szStr, const char *findstr, unsigned int iStart = 0);
		int InStr(const char *szStr, char c, unsigned int iStart = 0);

		// Mid - Copy part of a string
		char *Mid(const char *str, unsigned int start, unsigned int len = 0);

		// Left - take the left part of a string
		char *Left(const char *str, unsigned int len);

		// Right - take the right part of a string
		char *Right(const char *str, unsigned int len);

		// LCase - Convert string to lower case
		void LCase(char *str);

		// UCase - Convert string to upper case
		void UCase(char *str);

		// Str - Multiply string
		char *Str(const char *str, unsigned int count);
		char *Str(char c, unsigned int count);

		// Count - Count occurances of substring in string
		int Count(const char *str, const char *findstr);
		int Count(const char *str, char findchar);

		// Remove - Remove all occurances of a substring in a string
		char *Remove(const char *str, const char *remstr);
		char *Remove(const char *str, char remchar);

		// Replace - Replace all occurances of a substring in a string with another string
		char *Replace(const char *str, const char *findstr, const char *repstr);

		// ZeroMemory - Zero's out char pointer
		void ZeroMem(char *szStr, unsigned int iSize);

		// Len - Gets length of string
		unsigned int Len(const char *szStr);

		// Copy - copies source to dest
		void Copy(char **szDest, const char *szSource);
		void _Copy(char *szDest, const char *szSource);
		void _Copy(char *szDest, const char *szSource, unsigned int iSourceSize);

		// Concat - Concats szDest with szSource
		void Concat(char **szDest, const char *szSource);
		void _Concat(char *szDest, const char *szSource);
		//void _Concat(char *szDest, const char *szSource, unsigned int iSourceSize, unsigned int iDestSize);

		// ReplacePos - Replaces position in string with another string
		void ReplacePos(char **szStr, unsigned int iStart, unsigned int iLen, char *szReplaceStr);
		
		// Compare - Compares string with with another string, negative if string is less, 0 if equal, positive if greater
		signed int Compare(const char *szStr, const char *szStr2);
		// This function compares a number of chars from a string
		signed int Compare(const char *szStr, const char *szStr2, unsigned int iLen);
		// These functions ignores case when comparing
		signed int ICompare(const char *szStr, const char *szStr2);
		signed int ICompare(const char *szStr, const char *szStr2, unsigned int iLen);

	}

	// G::Vector
	template<class iTyp>
	class Vector
	{
	public:
		Vector();
		Vector(int size);
		Vector(int size, iTyp val);
		Vector(const Vector &v);
		Vector(int size, const iTyp ary[]);

		~Vector();

		const Vector & operator =(const Vector &v);

		int size() const;

		const iTyp & operator [](int i) const;
		iTyp & operator [](int i);

		void redim(int iSize);
		void redimp(int iSize);
		
		void push(const iTyp item);
		void pop(iTyp &item);
		
	private:
		int mySize;
		iTyp *myList;
	};

	
	// G::Queue
	template <class iTyp>
	struct gQueueNode
	{
		iTyp item;
		gQueueNode<iTyp> *next;
		gQueueNode<iTyp> *prev;
	};

	template <class iTyp>
	class Queue
	{
	public:
		Queue();
		Queue(const Queue<iTyp> &q);
		~Queue();

		int size(); // Return number of elements in queue
		void give(const iTyp i); // Put an element into the queue
		bool take(iTyp &i); // Remove an element from the queue
		void flush(); // Empty the queue
		bool peek(iTyp &i); // Take a look at the next element
	private:
		int iSize;
		gQueueNode<iTyp> *head;
		gQueueNode<iTyp> *AddNode();
	};

	// g::Stack
	template <class iTyp>
	struct gStackNode
	{
		iTyp item;
		gStackNode *next;
		//gStackNode *prev;
	};

	template <class iTyp>
	class Stack 
	{
	public:
		// Constructors/Destructor
		Stack();
		Stack(const Stack<iTyp> &s);
		~Stack();

		int size(); // Returns number of elements in stack
		void push(const iTyp i); // pushes element into stack
		bool pop(iTyp &i); // Pops top element off
		void flush(); // Clears out stack
	
	private:
		int iSize;
		gStackNode<iTyp> *head;
		gStackNode<iTyp> *AddNode();
	};


	namespace Token
	{
		// AddTok - Add a token to a string
		int AddTok(char **szTokenStr, const char *szAddToken, char cDelim);

		// DelTok - Delete Nth token from a string
		int DelTok(char **szTokenStr, unsigned int iDelTokenIndex, char cDelim);

		// FindTok - Find the position of the Nth matching token in a string
		int FindTok(const char *szTokenStr, const char *szFindToken, unsigned int iTokenIndex,
					char cDelim);
		
		// This function returns the total number of matching tokens
		int FindTok(const char *szTokenStr, const char *szFindToken, char cDelim);

		// GetTok - Get the Nth token in a string
		char *GetTok(const char *szTokenStr, unsigned int iTokenIndex, char cDelim);

		// InsTok - Inserts token into Nth position in a string
		int InsTok(char **szTokenStr, const char *szInsTokenStr, unsigned int iTokenIndex,
				   char cDelim);

		// IsTok - Checks to see if a token exists in a string
		bool IsTok(const char *szTokenStr, const char *szIsTokenStr, char cDelim);

		// MatchTok - Returns the Nth token that contains the specified string
		char *MatchTok(const char *szTokenStr, const char *szMatchTokenStr, 
					   int iTokenIndex, char cDelim);
		
		// This function counts the number of tokens which contain the specified string
		int MatchTok(const char *szTokenStr, const char *szMatchTokenStr,
					 char cDelim);

		// NumTok - Counts the number of tokens in a string
		int NumTok(const char *szTokenStr, char cDelim);

		// PutTok - Overwrites the Nth token in a string
		int PukTok(char **szTokenStr, const char *szPutTokenStr,
				   unsigned int iTokenIndex, char cDelim);

		// RemTok - Removes the Nth  token that contains the specified string in a string
		int RemTok(char **szTokenStr, const char *szMatchTokenStr,
				   unsigned int iTokenIndex, char cDelim);

		// RepTok - Replaces the Nth token that contains the specified string in a string
		int RepTok(char **szTokenStr, const char *szMatchTokenStr,
				   unsigned int iTokenIndex, char cDelim);

		// SortTok - Sorts the token in a string
		int SortTok(char **szTokenStr, unsigned int iSortFlags, char cDelim);

		// WildTok - Returns the Nth token that matches the wildcard string in a string
		char *WildTok(const char *szTokenStr, const char *szWildTokenStr,
					  unsigned int iTokenIndex, char cDelim);
		// This function returns the number of tokens that match the wildcard string
		int WildTok(const char *szTokenStr, const char *szWildTokenStr, char cDelim);

		// Tokenize - Returns a gVector containing all the tokens in a string
		void Tokenize(const char *szTokenStr, G::Vector<char *> &vszTokens, char cDelim);

		// UnTokenize - Returns a string from a vector of tokens
		void UnTokenize(G::Vector<char *> vszTokens, char **szTokenStr, char cDelim);
	}

	// G::Stuff
	namespace Stuff
	{
		template <class iTyp>
			G::Vector<iTyp> Sort(G::Vector<iTyp> vList);
		
		template <class iTyp>
			G::Vector<iTyp> Sort(G::Vector<iTyp> vList, int (*CompareFunction)(iTyp, iTyp));
		
		template <class iTyp>
			void Sort(Vector<iTyp> &vList, unsigned int iBegin, unsigned int iEnd);
		
		template <class iTyp>
			void Sort(Vector<iTyp> &vList, int (*CompareFunction)(iTyp, iTyp), 
			unsigned int iBegin, unsigned int iEnd);
		
		template <class iTyp>
			unsigned int Partition(Vector<iTyp> &vList, unsigned int iBegin, 
			unsigned int iEnd);
		
		template <class iTyp>
			unsigned int Partition(Vector<iTyp> &vList, 
			int (*CompareFunction)(iTyp, iTyp), unsigned int iBegin, unsigned int iEnd);
		
		template <class iTyp>
			void BubbleSort(Vector<iTyp> &vList, unsigned int iBegin,
			unsigned int iEnd);

		template <class iTyp>
			void BubbleSort(Vector<iTyp> &vList, int (*CompareFunction)(iTyp, iTyp),
			unsigned int iBegin, unsigned int iEnd);

		template <class iTyp>
			signed int Search(G::Vector<iTyp> vList, iTyp find);
		
		template <class iTyp>
			signed int Search(G::Vector<iTyp> vList, iTyp find, int (*CompareFunction)(iTyp, iTyp));
		
		template <class iTyp>
			signed int Search(G::Vector<iTyp> vList, iTyp find, unsigned int iBegin, unsigned int iEnd);
		
		template <class iTyp>
			signed int Search(G::Vector<iTyp> vList, iTyp find, int (*CompareFunction)(iTyp, iTyp), unsigned int iBegin, unsigned int iEnd);
		
		template <class iTyp>
			bool Reverse(G::Vector<iTyp> vList, G::Vector<iTyp> &vNewList);
		
		template <class iTyp>
			bool Reverse(G::Vector<iTyp> &vList);

	}
}

#include "gstring.h"
#include "gvector.h"
#include "gqueue.h"
#include "gstack.h"
#include "gtoken.h"
#include "gstuff.h"

#endif